import { XYWebSocket } from '~/utils/XYWebSocket'
import { isValidJson } from '~/utils'

export const useChatsStore = defineStore('chats', () => {
  const MESSAGE_TYPE = {
    text: 0,
    image: 1,
    audio: 2,
    video: 3,
    link: 4
  }
  const NOTICE_TYPE = []

  const userInfoStore = useUserInfoStore()
  const { userInfo } = storeToRefs(userInfoStore)

  let socket = null
  const initWebSocket = (url) => {
    if (!socket) {
      socket = new XYWebSocket.WebSocketClass(url)
      console.warn(socket, 'socket')
    }
  }
  const sendSocketMessage = async (message) => {
    return socket.sendMessage(message)
  }

  // 当前聊天会话管理

  const currConversationLoading = ref(false)
  /**
   *
   * @param userId 聊天对象Id
   * @returns {Promise<void>}
   */
  const getCurrConversationId = async (userId) => {
    try {
      return await request('/website/tourGroup/createMember', {
        method: 'post',
        body: {
          getUserId: userId,
          groupId: createGroupId()
        }
      })
    } catch (e) {
    } finally {
      currConversationLoading.value = false
    }
  }

  const getChatHistory = async (query) => {
    return await request('/website/tourMessage/getMessageByGroupId', { query })
  }

  const isRealMessage = (sendUserId) => {
    return sendUserId !== userInfo.value.pass
  }

  // 会话列表
  let chatList = ref([])
  let chatListLoading = ref(false)
  const getChatList = async () => {
    try {
      if (chatListLoading.value) return
      const res = await request('/website/tourism/fans/getTourMemberList')
      await handleResponse(res)
      const { list } = res.data
      chatList.value = list
    } catch (e) {
    } finally {
      chatListLoading.value = false
    }
  }

  // socket状态管理
  const STATUS_TEXT = ['连接关闭', '连接成功', '连接出错']
  let socketStatus = ref(0)
  let socketStatusText = computed(() => STATUS_TEXT[socketStatus.value])
  XYWebSocket.SocketEventsBus.on(XYWebSocket.SocketEvents.socketStatusChange, (status) => {
    socketStatus.value = status
  })

  const createGroupId = () => Date.now() + '' + Math.floor(Math.random() * 100000)

  // 处理后端接口回来的消息体
  // 修正sendUserId被后端修改
  const handleMessageList = (list) => {
    return (list ?? [])
      .filter((o) => isValidJson(o.messageContent))
      .map((o) => {
        let messageContent = JSON.parse(o.messageContent)
        messageContent.sendUserId = o.sendUserId
        messageContent.messageId = o.id
        messageContent.headImageUrl = o.headImageUrl
        messageContent.object.web_message_status = WEB_MESSAGE_STATUS.success // 后端接口过来的消息都标记成功
        // console.log(messageContent, 'messageContent')
        return messageContent
      })
  }

  const WEB_MESSAGE_STATUS = {
    loading: 'loading',
    success: 'success',
    error: 'error'
  }
  /**
   * 本地发送消息后，收到后端返回消息就更新本地消息 （写成hooks更好）
   * @param list
   * @param message
   * @returns {{index: number, message}}
   */
  const updateBackMessage = (list = [], message) => {
    const index = list.findIndex((o) => o.object?.id === message?.object?.id)
    if (index < 0) throw Error('')
    const status =
      message.messageLimit > 0 && message.messageLimit < 5
        ? WEB_MESSAGE_STATUS.error
        : WEB_MESSAGE_STATUS.success
    message.object.web_message_status = status

    return {
      index,
      message
    }
  }

  const getChatErrorText = (list = []) => {
    // const errorTextArr = ['','在对方关注或回复前，最多只能发送1条信息', '您已被踢出群', '该已被群解散', '该群已被封禁']
    const errorTextArr = [
      '',
      '在对方关注或回复前，最多只能发送1条信息',
      '您已被移除该群聊，或该群已解散',
      '您已被移除该群聊，或该群已解散',
      '该群已被封禁'
    ]
    const errorIndex = list.findIndex(
      (o) => o.object?.web_message_status === WEB_MESSAGE_STATUS.error
    )
    if (errorIndex < 0) return ''
    const errorMessage = list[errorIndex]
    return errorTextArr[errorMessage.messageLimit] ?? ''
  }

  return {
    // 注册socket
    initWebSocket,
    sendSocketMessage,

    // socket状态
    socketStatus,
    socketStatusText,

    // 当前会话
    getCurrConversationId,
    getChatHistory,

    // 会话列表
    chatList,
    chatListLoading,
    getChatList,

    // 排除自己发的消息
    isRealMessage,
    handleMessageList,

    WEB_MESSAGE_STATUS,
    updateBackMessage,

    getChatErrorText
  }
})
